(($, Drupal, once) => {
	/**
	 * Initialize autocomplete for glossary links.
	 */
	Drupal.behaviors.glossaryAutocomplete = {
		attach: (context, settings) => {
			for (const element of once(
				"glossary-autocomplete",
				"input.glossary-autocomplete",
				context,
			)) {
				const $input = $(element);

				$input.autocomplete({
					source: (request, response) => {
						$.ajax({
							url: `/glossary/autocomplete/${encodeURIComponent(request.term)}`,
							dataType: "json",
							success: (data) => {
								// Add "Create new term" option if user has permission
								if (
									Drupal.currentUser?.hasPermission?.(
										"create glossary terms via editor",
									) &&
									request.term.length >= 2
								) {
									data.push({
										id: "create_new",
										label: Drupal.t('Create new term: "@term"', {
											"@term": request.term,
										}),
										description: Drupal.t(
											"Click to create a new glossary term",
										),
										is_create_option: true,
										term_name: request.term,
									});
								}
								response(data);
							},
							error: () => {
								response([]);
							},
						});
					},
					minLength: 2,
					delay: 300,
					select: (event, ui) => {
						if (ui.item.is_create_option) {
							// Handle "Create new term" selection
							event.preventDefault();
							Drupal.glossaryAutocomplete.showCreateTermModal(
								ui.item.term_name,
								$input,
							);
							return false;
						}

						// Store the selected term ID
						$input.data("glossary-id", ui.item.id);
						// Display the term name with ID for clarity
						$input.val(`${ui.item.label} (${ui.item.id})`);
						return false;
					},
					focus: (event, ui) => {
						if (ui.item.is_create_option) {
							return false;
						}
						// Show term name during focus
						$input.val(ui.item.label);
						return false;
					},
					_renderItem: (ul, item) => {
						const $li = $("<li>");
						const $div = $("<div class='ui-menu-item-wrapper'>");

						if (item.is_create_option) {
							// Special styling for "Create new" option
							$div.addClass("create-new-option");
							$div.html(
								`<strong style='color: #0073aa;'>+ ${item.label}</strong><br><small>${item.description}</small>`,
							);
						} else {
							// Create the main content for regular terms
							let content = `<strong>${item.label}</strong>`;

							// Add language indicator if available
							if (item.language_name) {
								content += ` <span class='language-indicator'>${item.language_name}</span>`;
							}

							// Add description if available
							if (item.description) {
								content += `<br><small class='description'>${item.description}</small>`;
							}

							$div.html(content);
						}

						$li.append($div);
						return $li.appendTo(ul);
					},
				});
			}
		},
	};

	/**
	 * Glossary autocomplete utilities.
	 */
	Drupal.glossaryAutocomplete = Drupal.glossaryAutocomplete || {};

	/**
	 * Shows a modal for creating a new term.
	 */
	Drupal.glossaryAutocomplete.showCreateTermModal = (termName, $input) => {
		// Create modal HTML
		const modalHtml = `
      <div id="glossary-create-modal" class="glossary-modal-overlay">
        <div class="glossary-modal">
          <div class="glossary-modal-header">
            <h3>${Drupal.t("Create New Glossary Term")}</h3>
            <button type="button" class="glossary-modal-close">&times;</button>
          </div>
          <div class="glossary-modal-body">
            <div class="form-item">
              <label for="new-term-name">${Drupal.t("Term Name")} *</label>
              <input type="text" id="new-term-name" value="${termName}" required maxlength="255" />
            </div>
            <div class="form-item">
              <label for="new-term-description">${Drupal.t("Description")}</label>
              <textarea id="new-term-description" rows="3" maxlength="500" placeholder="${Drupal.t("Optional description for this term")}"></textarea>
            </div>
            <div class="glossary-modal-messages"></div>
          </div>
          <div class="glossary-modal-footer">
            <button type="button" class="btn btn-primary" id="create-term-submit">${Drupal.t("Create Term")}</button>
            <button type="button" class="btn btn-secondary glossary-modal-cancel">${Drupal.t("Cancel")}</button>
          </div>
        </div>
      </div>
    `;

		// Add modal to page
		$("body").append(modalHtml);
		const $modal = $("#glossary-create-modal");

		// Store current scroll position and prevent body scrolling
		const scrollTop = $(window).scrollTop();
		$("body").addClass("glossary-modal-open").css("top", `${-scrollTop}px`);
		$("body").data("scroll-top", scrollTop);

		// Focus on name field
		$("#new-term-name").focus().select();

		// Prevent modal from closing when clicking inside the modal content
		$modal.find(".glossary-modal").on("click", (e) => {
			e.stopPropagation();
		});

		// Function to close modal and restore body state
		const closeModal = () => {
			const scrollTop = $("body").data("scroll-top") || 0;
			$("body")
				.removeClass("glossary-modal-open")
				.css("top", "")
				.removeData("scroll-top");
			$(window).scrollTop(scrollTop);
			$modal.remove();
		};

		// Handle close buttons
		$modal
			.find(".glossary-modal-close, .glossary-modal-cancel")
			.on("click", closeModal);

		$modal.on("click", function (e) {
			// Only close if clicking directly on the overlay (not on modal content)
			if (e.target === this) {
				closeModal();
			}
		});

		// Handle form submission
		$("#create-term-submit").on("click", function () {
			const name = $("#new-term-name").val().trim();
			const description = $("#new-term-description").val().trim();

			if (!name) {
				Drupal.glossaryAutocomplete.showModalMessage(
					Drupal.t("Term name is required"),
					"error",
				);
				return;
			}

			// Disable button and show loading
			$(this).prop("disabled", true).text(Drupal.t("Creating..."));

			// Create the term
			$.ajax({
				url: "/glossary/create-term",
				method: "POST",
				contentType: "application/json",
				data: JSON.stringify({
					name: name,
					description: description,
				}),
				success: (response) => {
					if (response.success && response.term) {
						// Set the new term in the input
						$input.data("glossary-id", response.term.id);
						$input.val(`${response.term.name} (${response.term.id})`);

						// Close modal
						closeModal();

						// Get the CKEditor instance and execute the glossaryLink command
						const editor = $input.data("ckeditor-instance");
						const uiInstance = $input.data("ui-instance");

						if (editor && uiInstance) {
							// Execute the glossaryLink command to create the link
							editor.execute("glossaryLink", response.term.id);
							// Hide the form
							uiInstance._hideForm();
						}

						// Show success message
						Drupal.announce(
							Drupal.t(
								'Glossary term "@name" created and linked successfully.',
								{ "@name": response.term.name },
							),
						);
					} else {
						Drupal.glossaryAutocomplete.showModalMessage(
							Drupal.t("Failed to create term"),
							"error",
						);
					}
				},
				error: (xhr) => {
					let errorMsg = Drupal.t("Failed to create term");
					if (xhr.responseJSON?.error) {
						errorMsg = xhr.responseJSON.error;

						// Handle existing term case
						if (xhr.status === 409 && xhr.responseJSON.existing_term) {
							const existingTerm = xhr.responseJSON.existing_term;
							errorMsg += Drupal.t(". Use existing term?");

							const useExistingHtml = `
                <div class="existing-term-option">
                  <button type="button" class="btn btn-link" id="use-existing-term" 
                          data-term-id="${existingTerm.id}" data-term-name="${existingTerm.name}">
                    ${Drupal.t('Use "@name" (ID: @id)', { "@name": existingTerm.name, "@id": existingTerm.id })}
                  </button>
                </div>
              `;

							$(".glossary-modal-messages").append(useExistingHtml);

							$("#use-existing-term").on("click", function () {
								const termId = $(this).data("term-id");
								const termName = $(this).data("term-name");

								$input.data("glossary-id", termId);
								$input.val(`${termName} (${termId})`);
								closeModal();

								// Get the CKEditor instance and execute the glossaryLink command
								const editor = $input.data("ckeditor-instance");
								const uiInstance = $input.data("ui-instance");

								if (editor && uiInstance) {
									// Execute the glossaryLink command to create the link
									editor.execute("glossaryLink", termId);
									// Hide the form
									uiInstance._hideForm();
								}
							});
						}
					}

					Drupal.glossaryAutocomplete.showModalMessage(errorMsg, "error");
				},
				complete: () => {
					$("#create-term-submit")
						.prop("disabled", false)
						.text(Drupal.t("Create Term"));
				},
			});
		});

		// Handle Enter key in form
		$modal.on("keypress", "input, textarea", (e) => {
			if (e.which === 13 && !e.shiftKey) {
				e.preventDefault();
				$("#create-term-submit").click();
			}
		});
	};

	/**
	 * Shows a message in the modal.
	 */
	Drupal.glossaryAutocomplete.showModalMessage = (message, type) => {
		const $messages = $(".glossary-modal-messages");
		$messages.empty();

		const alertClass = type === "error" ? "alert-danger" : "alert-success";
		const messageHtml = `<div class="alert ${alertClass}">${message}</div>`;

		$messages.html(messageHtml);

		// Auto-hide success messages
		if (type === "success") {
			setTimeout(() => {
				$messages.empty();
			}, 3000);
		}
	};
})(jQuery, Drupal, once);
